@@ -11,6 +11,10 @@ DECLSPEC u32 h_fun128 (const u32 x, const u32 *key);
 DECLSPEC void twofish128_set_key (u32 *sk, u32 *lk, const u32 *ukey);
 DECLSPEC void twofish128_encrypt (const u32 *sk, const u32 *lk, const u32 *in, u32 *out);
 DECLSPEC void twofish128_decrypt (const u32 *sk, const u32 *lk, const u32 *in, u32 *out);
+DECLSPEC u32 h_fun192 (const u32 x, const u32 *key);
+DECLSPEC void twofish192_set_key (u32 *sk, u32 *lk, const u32 *ukey);
+DECLSPEC void twofish192_encrypt (const u32 *sk, const u32 *lk, const u32 *in, u32 *out);
+DECLSPEC void twofish192_decrypt (const u32 *sk, const u32 *lk, const u32 *in, u32 *out);
 DECLSPEC u32 h_fun256 (const u32 x, const u32 *key);
 DECLSPEC void twofish256_set_key (u32 *sk, u32 *lk, const u32 *ukey);
 DECLSPEC void twofish256_encrypt (const u32 *sk, const u32 *lk, const u32 *in, u32 *out);
