@@ -172,6 +172,14 @@ u32 brain_compute_attack (hashcat_ctx_t *hashcat_ctx)
 
   XXH64_update (state, &hex_salt, sizeof (hex_salt));
 
+  const u32 opti_type = hashconfig->opti_type;
+
+  XXH64_update (state, &opti_type, sizeof (opti_type));
+
+  const u64 opts_type = hashconfig->opts_type;
+
+  XXH64_update (state, &opts_type, sizeof (opts_type));
+
   const int hccapx_message_pair = user_options->hccapx_message_pair;
 
   XXH64_update (state, &hccapx_message_pair, sizeof (hccapx_message_pair));
@@ -295,10 +303,12 @@ u32 brain_compute_attack (hashcat_ctx_t *hashcat_ctx)
 
     const int markov_classic   = user_options->markov_classic;
     const int markov_disable   = user_options->markov_disable;
+    const int markov_inverse   = user_options->markov_inverse;
     const int markov_threshold = user_options->markov_threshold;
 
     XXH64_update (state, &markov_classic,   sizeof (markov_classic));
     XXH64_update (state, &markov_disable,   sizeof (markov_disable));
+    XXH64_update (state, &markov_inverse,   sizeof (markov_inverse));
     XXH64_update (state, &markov_threshold, sizeof (markov_threshold));
 
     if (user_options->markov_hcstat2)
@@ -352,10 +362,12 @@ u32 brain_compute_attack (hashcat_ctx_t *hashcat_ctx)
 
     const int markov_classic   = user_options->markov_classic;
     const int markov_disable   = user_options->markov_disable;
+    const int markov_inverse   = user_options->markov_inverse;
     const int markov_threshold = user_options->markov_threshold;
 
     XXH64_update (state, &markov_classic,   sizeof (markov_classic));
     XXH64_update (state, &markov_disable,   sizeof (markov_disable));
+    XXH64_update (state, &markov_inverse,   sizeof (markov_inverse));
     XXH64_update (state, &markov_threshold, sizeof (markov_threshold));
 
     if (user_options->markov_hcstat2)
@@ -445,10 +457,12 @@ u32 brain_compute_attack (hashcat_ctx_t *hashcat_ctx)
 
     const int markov_classic   = user_options->markov_classic;
     const int markov_disable   = user_options->markov_disable;
+    const int markov_inverse   = user_options->markov_inverse;
     const int markov_threshold = user_options->markov_threshold;
 
     XXH64_update (state, &markov_classic,   sizeof (markov_classic));
     XXH64_update (state, &markov_disable,   sizeof (markov_disable));
+    XXH64_update (state, &markov_inverse,   sizeof (markov_inverse));
     XXH64_update (state, &markov_threshold, sizeof (markov_threshold));
 
     if (user_options->markov_hcstat2)
@@ -522,6 +536,46 @@ u32 brain_compute_attack (hashcat_ctx_t *hashcat_ctx)
       XXH64_update (state, rule_buf_r, strlen (rule_buf_r));
     }
   }
+  else if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
+  {
+    if (straight_ctx->dict)
+    {
+      const u64 wordlist_hash = brain_compute_attack_wordlist (straight_ctx->dict);
+
+      XXH64_update (state, &wordlist_hash, sizeof (wordlist_hash));
+    }
+
+    const int hex_wordlist = user_options->hex_wordlist;
+
+    XXH64_update (state, &hex_wordlist, sizeof (hex_wordlist));
+
+    const int wordlist_autohex_disable = user_options->wordlist_autohex_disable;
+
+    XXH64_update (state, &wordlist_autohex_disable, sizeof (wordlist_autohex_disable));
+
+    if (user_options->encoding_from)
+    {
+      const char *encoding_from = user_options->encoding_from;
+
+      XXH64_update (state, encoding_from, strlen (encoding_from));
+    }
+
+    if (user_options->encoding_to)
+    {
+      const char *encoding_to = user_options->encoding_to;
+
+      XXH64_update (state, encoding_to, strlen (encoding_to));
+    }
+
+    if (user_options->rule_buf_l)
+    {
+      const char *rule_buf_l = user_options->rule_buf_l;
+
+      XXH64_update (state, rule_buf_l, strlen (rule_buf_l));
+    }
+
+    XXH64_update (state, straight_ctx->kernel_rules_buf, straight_ctx->kernel_rules_cnt * sizeof (kernel_rule_t));
+  }
 
   const u32 brain_attack = (const u32) XXH64_digest (state);
 
@@ -612,7 +666,7 @@ u32 brain_auth_challenge (void)
 
   #else
 
-  static const char *urandom = "/dev/urandom";
+  static const char *const urandom = "/dev/urandom";
 
   HCFILE fp;
 
@@ -2205,6 +2259,8 @@ void *brain_server_handle_client (void *p)
 
       brain_server_dbs->client_slots[client_idx] = 0;
 
+      hc_thread_mutex_unlock (brain_server_dbs->mux_dbs);
+
       close (client_fd);
 
       return NULL;
@@ -2239,6 +2295,8 @@ void *brain_server_handle_client (void *p)
 
       brain_server_dbs->client_slots[client_idx] = 0;
 
+      hc_thread_mutex_unlock (brain_server_dbs->mux_dbs);
+
       close (client_fd);
 
       return NULL;
@@ -2278,6 +2336,10 @@ void *brain_server_handle_client (void *p)
   {
     brain_logging (stderr, 0, "%s\n", MSG_ENOMEM);
 
+    brain_server_dbs->client_slots[client_idx] = 0;
+
+    close (client_fd);
+
     return NULL;
   }
 
@@ -3312,3 +3374,38 @@ int brain_server (const char *listen_host, const int listen_port, const char *br
 
   return 0;
 }
+
+int brain_ctx_init (hashcat_ctx_t *hashcat_ctx)
+{
+  brain_ctx_t    *brain_ctx    = hashcat_ctx->brain_ctx;
+  user_options_t *user_options = hashcat_ctx->user_options;
+
+  #ifdef WITH_BRAIN
+  brain_ctx->support = true;
+  #else
+  brain_ctx->support = false;
+  #endif
+
+  if (brain_ctx->support == false) return 0;
+
+  if (user_options->brain_client == true)
+  {
+    brain_ctx->enabled = true;
+  }
+
+  if (user_options->brain_server == true)
+  {
+    brain_ctx->enabled = true;
+  }
+
+  return 0;
+}
+
+void brain_ctx_destroy (hashcat_ctx_t *hashcat_ctx)
+{
+  brain_ctx_t *brain_ctx = hashcat_ctx->brain_ctx;
+
+  if (brain_ctx->support == false) return;
+
+  memset (brain_ctx, 0, sizeof (brain_ctx_t));
+}
