@@ -19,7 +19,7 @@ int combinator_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   combinator_ctx->enabled = false;
 
-  if (user_options->example_hashes == true) return 0;
+  if (user_options->hash_info      == true) return 0;
   if (user_options->left           == true) return 0;
   if (user_options->backend_info   == true) return 0;
   if (user_options->show           == true) return 0;
