@@ -16,7 +16,7 @@
 #include "ext_lzma.h"
 #include "mpsp.h"
 
-static const char *DEF_MASK = "?1?2?2?2?2?2?2?3?3?3?3?d?d?d?d";
+static const char *const DEF_MASK = "?1?2?2?2?2?2?2?3?3?3?3?d?d?d?d";
 
 #define MAX_MFS 5 // 4*charset, 1*mask
 
@@ -659,6 +659,7 @@ static int sp_setup_tbl (hashcat_ctx_t *hashcat_ctx)
   char *hcstat  = user_options->markov_hcstat2;
   u32   disable = user_options->markov_disable;
   u32   classic = user_options->markov_classic;
+  bool  inverse = user_options->markov_inverse;
 
   hcstat_table_t *root_table_buf   = mask_ctx->root_table_buf;
   hcstat_table_t *markov_table_buf = mask_ctx->markov_table_buf;
@@ -720,7 +721,7 @@ static int sp_setup_tbl (hashcat_ctx_t *hashcat_ctx)
 
   HCFILE fp;
 
-  if (hc_fopen (&fp, hcstat, "rb") == false)
+  if (hc_fopen_raw (&fp, hcstat, "rb") == false)
   {
     event_log_error (hashcat_ctx, "%s: %s", hcstat, strerror (errno));
 
@@ -793,6 +794,16 @@ static int sp_setup_tbl (hashcat_ctx_t *hashcat_ctx)
   for (int i = 0; i < SP_ROOT_CNT; i++)   root_stats_buf[i]   = byte_swap_64 (root_stats_buf[i]);
   for (int i = 0; i < SP_MARKOV_CNT; i++) markov_stats_buf[i] = byte_swap_64 (markov_stats_buf[i]);
 
+  /**
+   * markov inverse: https://github.com/hashcat/hashcat/issues/1058
+   */
+
+  if (inverse == true)
+  {
+    for (int i = 0; i < SP_ROOT_CNT; i++)   root_stats_buf[i]   = 0 - (1 + root_stats_buf[i]);
+    for (int i = 0; i < SP_MARKOV_CNT; i++) markov_stats_buf[i] = 0 - (1 + markov_stats_buf[i]);
+  }
+
   /**
    * verify header
    */
@@ -1391,7 +1402,7 @@ int mask_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   mask_ctx->enabled = false;
 
-  if (user_options->example_hashes == true) return 0;
+  if (user_options->hash_info      == true) return 0;
   if (user_options->left           == true) return 0;
   if (user_options->backend_info   == true) return 0;
   if (user_options->show           == true) return 0;
@@ -1400,6 +1411,7 @@ int mask_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   if (user_options->attack_mode == ATTACK_MODE_STRAIGHT) return 0;
   if (user_options->attack_mode == ATTACK_MODE_COMBI)    return 0;
+  if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)  return 0;
 
   mask_ctx->enabled = true;
 
