@@ -163,7 +163,7 @@ static int outfile_remove (hashcat_ctx_t *hashcat_ctx)
 
       struct stat outfile_stat;
 
-      if (fstat (hc_fileno (&fp), &outfile_stat))
+      if (hc_fstat (&fp, &outfile_stat))
       {
         hc_fclose (&fp);
 
@@ -224,22 +224,7 @@ static int outfile_remove (hashcat_ctx_t *hashcat_ctx)
             memset (hash_buf.hook_salt, 0, hashconfig->hook_salt_size);
           }
 
-          int parser_status = PARSER_HASH_LENGTH;
-
-          if (module_ctx->module_hash_decode_potfile != MODULE_DEFAULT)
-          {
-            void *tmps = hcmalloc (hashconfig->tmp_size);
-
-            parser_status = module_ctx->module_hash_decode_potfile (hashconfig, hash_buf.digest, hash_buf.salt, hash_buf.esalt, hash_buf.hook_salt, hash_buf.hash_info, line_buf, line_hash_len, tmps);
-
-            hcfree (tmps);
-          }
-          else
-          {
-            // "normal" case: hash in the outfile is the same as the hash in the original hash file
-
-            parser_status = module_ctx->module_hash_decode (hashconfig, hash_buf.digest, hash_buf.salt, hash_buf.esalt, hash_buf.hook_salt, hash_buf.hash_info, line_buf, line_hash_len);
-          }
+          int parser_status = module_ctx->module_hash_decode (hashconfig, hash_buf.digest, hash_buf.salt, hash_buf.esalt, hash_buf.hook_salt, hash_buf.hash_info, line_buf, line_hash_len);
 
           if (parser_status != PARSER_OK) continue;
 
@@ -358,12 +343,13 @@ int outcheck_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   outcheck_ctx->enabled = false;
 
-  if (user_options->keyspace       == true) return 0;
-  if (user_options->benchmark      == true) return 0;
-  if (user_options->example_hashes == true) return 0;
-  if (user_options->speed_only     == true) return 0;
-  if (user_options->progress_only  == true) return 0;
-  if (user_options->backend_info   == true) return 0;
+  if (user_options->keyspace      == true) return 0;
+  if (user_options->benchmark     == true) return 0;
+  if (user_options->hash_info     == true) return 0;
+  if (user_options->speed_only    == true) return 0;
+  if (user_options->progress_only == true) return 0;
+  if (user_options->backend_info  == true) return 0;
+  if (user_options->identify      == true) return 0;
 
   if (hashconfig->outfile_check_disable == true) return 0;
 
