@@ -13,15 +13,6 @@
 #include "folder.h"
 #include "restore.h"
 
-#if defined (_WIN)
-static void fsync (int fd)
-{
-  HANDLE h = (HANDLE) _get_osfhandle (fd);
-
-  FlushFileBuffers (h);
-}
-#endif
-
 static int init_restore (hashcat_ctx_t *hashcat_ctx)
 {
   restore_ctx_t *restore_ctx = hashcat_ctx->restore_ctx;
@@ -232,7 +223,7 @@ static int write_restore (hashcat_ctx_t *hashcat_ctx)
 
   hc_fflush (&fp);
 
-  fsync (hc_fileno (&fp));
+  hc_fsync (&fp);
 
   hc_fclose (&fp);
 
@@ -299,7 +290,7 @@ int restore_ctx_init (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
   restore_ctx->enabled = false;
 
   if (user_options->benchmark       == true) return 0;
-  if (user_options->example_hashes  == true) return 0;
+  if (user_options->hash_info       == true) return 0;
   if (user_options->keyspace        == true) return 0;
   if (user_options->left            == true) return 0;
   if (user_options->backend_info    == true) return 0;
@@ -309,6 +300,7 @@ int restore_ctx_init (hashcat_ctx_t *hashcat_ctx, int argc, char **argv)
   if (user_options->progress_only   == true) return 0;
   if (user_options->usage           == true) return 0;
   if (user_options->version         == true) return 0;
+  if (user_options->identify        == true) return 0;
   if (user_options->restore_disable == true) return 0;
 
   if (argc ==    0) return 0;
