@@ -18,9 +18,9 @@ static int straight_ctx_add_wl (hashcat_ctx_t *hashcat_ctx, const char *dict)
 {
   if (hc_path_has_bom (dict) == true)
   {
-    event_log_error (hashcat_ctx, "%s: Byte Order Mark (BOM) was detected", dict);
+    event_log_warning (hashcat_ctx, "%s: Byte Order Mark (BOM) was detected", dict);
 
-    return -1;
+    //return -1;
   }
 
   straight_ctx_t *straight_ctx = hashcat_ctx->straight_ctx;
@@ -43,6 +43,7 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 {
   combinator_ctx_t     *combinator_ctx     = hashcat_ctx->combinator_ctx;
   induct_ctx_t         *induct_ctx         = hashcat_ctx->induct_ctx;
+  hashes_t             *hashes             = hashcat_ctx->hashes;
   logfile_ctx_t        *logfile_ctx        = hashcat_ctx->logfile_ctx;
   mask_ctx_t           *mask_ctx           = hashcat_ctx->mask_ctx;
   status_ctx_t         *status_ctx         = hashcat_ctx->status_ctx;
@@ -200,6 +201,55 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
       return 0;
     }
   }
+  else if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
+  {
+    if (user_options_extra->wordlist_mode == WL_MODE_FILE)
+    {
+      straight_ctx->dict = straight_ctx->dicts[straight_ctx->dicts_pos];
+
+      logfile_sub_string (straight_ctx->dict);
+
+      for (u32 i = 0; i < user_options->rp_files_cnt; i++)
+      {
+        logfile_sub_var_string ("rulefile", user_options->rp_files[i]);
+      }
+
+      HCFILE fp;
+
+      if (hc_fopen (&fp, straight_ctx->dict, "rb") == false)
+      {
+        event_log_error (hashcat_ctx, "%s: %s", straight_ctx->dict, strerror (errno));
+
+        return -1;
+      }
+
+      const int rc = count_words (hashcat_ctx, &fp, straight_ctx->dict, &status_ctx->words_cnt);
+
+      hc_fclose (&fp);
+
+      if (rc == -1)
+      {
+        event_log_error (hashcat_ctx, "Integer overflow detected in keyspace of wordlist: %s", straight_ctx->dict);
+
+        return -1;
+      }
+
+      if ((status_ctx->words_cnt / straight_ctx->kernel_rules_cnt) != hashes->salts_cnt)
+      {
+        event_log_error (hashcat_ctx, "Number of words in wordlist '%s' is not in sync with number of unique salts", straight_ctx->dict);
+        event_log_error (hashcat_ctx, "Words: %" PRIu64 ", salts: %d", status_ctx->words_cnt / straight_ctx->kernel_rules_cnt, hashes->salts_cnt);
+
+        return -1;
+      }
+
+      if (status_ctx->words_cnt == 0)
+      {
+        logfile_sub_msg ("STOP");
+
+        return 0;
+      }
+    }
+  }
 
   return 0;
 }
@@ -212,7 +262,7 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
 
   straight_ctx->enabled = false;
 
-  if (user_options->example_hashes == true) return 0;
+  if (user_options->hash_info      == true) return 0;
   if (user_options->left           == true) return 0;
   if (user_options->backend_info   == true) return 0;
   if (user_options->show           == true) return 0;
@@ -243,7 +293,7 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
     }
     else if (user_options->rp_gen)
     {
-      if (kernel_rules_generate (hashcat_ctx, &straight_ctx->kernel_rules_buf, &straight_ctx->kernel_rules_cnt) == -1) return -1;
+      if (kernel_rules_generate (hashcat_ctx, &straight_ctx->kernel_rules_buf, &straight_ctx->kernel_rules_cnt, user_options->rp_gen_func_sel) == -1) return -1;
     }
   }
 
@@ -251,7 +301,7 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
    * wordlist based work
    */
 
-  if (user_options->attack_mode == ATTACK_MODE_STRAIGHT)
+  if ((user_options->attack_mode == ATTACK_MODE_STRAIGHT) || (user_options->attack_mode == ATTACK_MODE_ASSOCIATION))
   {
     if (user_options_extra->wordlist_mode == WL_MODE_FILE)
     {
